'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

var forceUpdateReducer = function forceUpdateReducer(state) {
  return state + 1;
}; // For server-side rendering: https://github.com/react-spring/zustand/pull/34


var useIsoLayoutEffect = typeof window === 'undefined' ? react.useEffect : react.useLayoutEffect;
function create(createState) {
  // All listeners are wrapped in a function with the signature: () => void
  var listeners = [];
  var state;
  var renderCount = 0; // Returns an int for a component based on render order.

  function useRenderId() {
    var renderIdRef = react.useRef();

    if (!renderIdRef.current) {
      renderIdRef.current = renderCount++;
    }

    return renderIdRef.current;
  }

  var setState = function setState(partial) {
    var partialState = typeof partial === 'function' ? partial(state) : partial;

    if (partialState !== state) {
      state = Object.assign({}, state, partialState);
      listeners.forEach(function (listener) {
        return listener();
      });
    }
  };

  var getState = function getState() {
    return state;
  };

  var subscribe = function subscribe(listener, options) {
    if (options === void 0) {
      options = {};
    }

    if (!('currentSlice' in options)) {
      options.currentSlice = (options.selector || getState)(state);
    } // subscribe can be called externally without passing in a listenerIndex so
    // we need to assign it a default index.


    var _options = options,
        _options$listenerInde = _options.listenerIndex,
        listenerIndex = _options$listenerInde === void 0 ? renderCount++ : _options$listenerInde;

    var listenerWrapper = function listenerWrapper() {
      // Access the current values of the options object in listenerWrapper.
      // We rely on this because options is mutated in useStore.
      var _options2 = options,
          _options2$selector = _options2.selector,
          selector = _options2$selector === void 0 ? getState : _options2$selector,
          _options2$equalityFn = _options2.equalityFn,
          equalityFn = _options2$equalityFn === void 0 ? Object.is : _options2$equalityFn; // Selector or equality function could throw but we don't want to stop
      // the listener from being called.
      // https://github.com/react-spring/zustand/pull/37

      try {
        var newStateSlice = selector(state);

        if (!equalityFn(options.currentSlice, newStateSlice)) {
          listener(options.currentSlice = newStateSlice);
        }
      } catch (error) {
        options.subscribeError = error;
        listener(null, error);
      }
    };

    listeners[listenerIndex] = listenerWrapper; // Intentially using delete because shortening the length of the listeners
    // array would result in listenerIndex not accessing the correct listener.
    // This means listeners should be considered a sparce array.

    return function () {
      return delete listeners[listenerIndex];
    };
  };

  var apiSubscribe = function apiSubscribe(listener, selector, equalityFn) {
    return subscribe(listener, {
      selector: selector,
      equalityFn: equalityFn
    });
  };

  var destroy = function destroy() {
    return listeners.length = 0;
  };

  var useStore = function useStore(selector, equalityFn) {
    if (selector === void 0) {
      selector = getState;
    }

    if (equalityFn === void 0) {
      equalityFn = Object.is;
    }

    var listenerIndex = useRenderId();
    var optionsRef = react.useRef();

    if (!optionsRef.current) {
      optionsRef.current = {
        selector: selector,
        equalityFn: equalityFn,
        currentSlice: selector(state),
        listenerIndex: listenerIndex
      };
    }

    var options = optionsRef.current; // Update state slice if selector has changed or subscriber errored.

    if (selector !== options.selector || options.subscribeError) {
      var newStateSlice = selector(state);

      if (!equalityFn(options.currentSlice, newStateSlice)) {
        options.currentSlice = newStateSlice;
      }
    }

    useIsoLayoutEffect(function () {
      options.selector = selector;
      options.equalityFn = equalityFn;
      options.subscribeError = undefined;
    });
    var forceUpdate = react.useReducer(forceUpdateReducer, 0)[1];
    useIsoLayoutEffect(function () {
      return subscribe(forceUpdate, options);
    }, []);
    return options.currentSlice;
  };

  var api = {
    setState: setState,
    getState: getState,
    subscribe: apiSubscribe,
    destroy: destroy
  };
  state = createState(setState, getState, api);
  return [useStore, api];
}

exports.create = create;
exports.default = create;
